#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "odicon.h"

static void help(int argc, char *argv[]) {
    printf("usage: %s opt dim [seed]\n", argv[0]);
    printf("opt: L  = L-convex minimize\n"
           "     M  = M-convex minimize\n"
           "     S  = Submodular minimize\n"
           "     QL = Quasi L-convex minimize\n"
           "     2  = quadratic\n"
           "     V  = Validate L-/M-convex\n"
           );
}

int main(int argc, char *argv[]) {
    int dim, seed = 12345;

    if (argc <= 1) {  /* no arg */
        help(argc, argv);
        return EXIT_SUCCESS;
    }

    if (argc <= 2) {
        fprintf(stderr, "Give me dimension.\n");
        return EXIT_FAILURE;
    }

    dim = atoi(argv[2]);
    if (argc > 3) seed = atoi(argv[3]);

    if (toupper(argv[1][0]) == 'D') {
        with_limit = TRUE;
    } else {
        printf("%s: dim = %d, seed = %d\n", argv[0], dim, seed);
    }
    mt_initialize(seed);
    cpu_time();
#define HILO(x,y) ((x)<<8|(y))

    switch (HILO(toupper(argv[1][0]),toupper(argv[1][1]))) {
    case HILO('L',0):   test_lconv2(dim, argc, argv); break;
    case HILO('M',0):   test_mconv(dim);  break;
    case HILO('S',0):   test_submodular(dim); break;
    case HILO('Q','L'): test_qlconv(dim); break;
    case HILO('2',0):   test_quadratic(dim); break;
    case HILO('V',0):   test_valid(dim); break;

    case HILO('D','L'): demo_Lseparable(dim, toupper(argv[1][2])); break;
    case HILO('D','M'): demo_Mseparable(dim, toupper(argv[1][2])); break;
    case HILO('D','Q'): demo_quadratic(dim,  toupper(argv[1][2])); break;
    case HILO('D','Z'): demo_inventory(dim);  break;
    case HILO('D','S'): demo_shift_schedule(dim);  break;
    default:
        help(argc, argv);
        break;
    }
    return EXIT_SUCCESS;
}
